"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createGlobeLayer = createGlobeLayer;
exports.default = exports.GLOBE_VIEW_EVENTS = void 0;

var THREE = _interopRequireWildcard(require("three"));

var _View = _interopRequireWildcard(require("../View"));

var _MainLoop = require("../MainLoop");

var _ColorLayersOrdering = require("../../Renderer/ColorLayersOrdering");

var _GlobeControls = _interopRequireDefault(require("../../Controls/GlobeControls"));

var _GlobeLayer = _interopRequireDefault(require("./Globe/GlobeLayer"));

var _Atmosphere = _interopRequireDefault(require("./Globe/Atmosphere"));

var _CoordStars = _interopRequireDefault(require("../Geographic/CoordStars"));

var _Coordinates = _interopRequireWildcard(require("../Geographic/Coordinates"));

/**
 * Fires when the view is completely loaded. Controls and view's functions can be called then.
 * @event GlobeView#initialized
 * @property target {view} dispatched on view
 * @property type {string} initialized
 */

/**
 * Fires when a layer is added
 * @event GlobeView#layer-added
 * @property layerId {string} the id of the layer
 * @property target {view} dispatched on view
 * @property type {string} layers-added
 */

/**
 * Fires when a layer is removed
 * @event GlobeView#layer-removed
 * @property layerId {string} the id of the layer
 * @property target {view} dispatched on view
 * @property type {string} layers-added
 */

/**
 * Fires when the layers oder has changed
 * @event GlobeView#layers-order-changed
 * @property new {object}
 * @property new.sequence {array}
 * @property new.sequence.0 {number} the new layer at position 0
 * @property new.sequence.1 {number} the new layer at position 1
 * @property new.sequence.2 {number} the new layer at position 2
 * @property previous {object}
 * @property previous.sequence {array}
 * @property previous.sequence.0 {number} the previous layer at position 0
 * @property previous.sequence.1 {number} the previous layer at position 1
 * @property previous.sequence.2 {number} the previous layer at position 2
 * @property target {view} dispatched on view
 * @property type {string} layers-order-changed
 */

/**
 * Globe's EVENT
 * @property GLOBE_INITIALIZED {string} emit one time when globe is initialized
 * @property LAYER_ADDED {string} emit when layer id added in viewer
 * @property LAYER_REMOVED {string} emit when layer id removed in viewer
 * @property COLOR_LAYERS_ORDER_CHANGED {string} emit when  color layers order change
 */
var GLOBE_VIEW_EVENTS = {
  GLOBE_INITIALIZED: 'initialized',
  LAYER_ADDED: 'layer-added',
  LAYER_REMOVED: 'layer-removed',
  COLOR_LAYERS_ORDER_CHANGED: _ColorLayersOrdering.COLOR_LAYERS_ORDER_CHANGED
};
exports.GLOBE_VIEW_EVENTS = GLOBE_VIEW_EVENTS;

function createGlobeLayer(id) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  console.warn('createGlobeLayer is deprecated, use the GlobeLayer class instead.');
  return new _GlobeLayer.default(id, options.object3d, options);
}
/**
 * Creates a view of a globe.
 *
 * @constructor
 *
 * @example
 * var viewerDiv = document.getElementById('viewerDiv');
 * var position = new itowns.Coordinates('WGS84', 2.35, 48.8, 25e6);
 * var view = new itowns.GlobeView(viewerDiv, position);
 *
 * @example
 * var viewerDiv = document.getElementById('viewerDiv');
 * var position = { longitude: 2.35, latitude: 48.8, altitude: 25e6 };
 * var view = new itowns.GlobeView(viewerDiv, position);
 *
 * @param {HTMLDivElement} viewerDiv - Where to attach the view and display it
 * in the DOM.
 * @param {object|Coordinates} coordCarto - An object containing three
 * properties: longitude, latitude and altitude. It will help placing the camera
 * on the globe at the creation.
 * @param {object=} options - See options of {@link View}.
 */


function GlobeView(viewerDiv, coordCarto) {
  var _this = this;

  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  THREE.Object3D.DefaultUp.set(0, 0, 1); // Setup View

  _View.default.call(this, 'EPSG:4978', viewerDiv, options); // Configure camera


  var positionCamera;

  if (coordCarto instanceof _Coordinates.default) {
    positionCamera = coordCarto.as('EPSG:4326');
  } else {
    positionCamera = new _Coordinates.C.EPSG_4326(coordCarto.longitude, coordCarto.latitude, coordCarto.altitude);
  }

  this.camera.camera3D.near = Math.max(15.0, 0.000002352 * _Coordinates.ellipsoidSizes.x);
  this.camera.camera3D.far = _Coordinates.ellipsoidSizes.x * 10;
  var tileLayer = new _GlobeLayer.default('globe', options.object3d, options);
  var sun = new THREE.DirectionalLight();
  sun.position.set(-0.5, 0, 1);
  sun.updateMatrixWorld(true);
  tileLayer.object3d.add(sun);
  this.addLayer(tileLayer); // Atmosphere

  this.atmosphere = new _Atmosphere.default();
  var atmosphereLayer = this.mainLoop.gfxEngine.getUniqueThreejsLayer();
  this.atmosphere.traverse(function (obj) {
    obj.layers.set(atmosphereLayer);
  });
  this.camera.camera3D.layers.enable(atmosphereLayer);
  tileLayer.object3d.add(this.atmosphere);
  this.atmosphere.updateMatrixWorld(true); // Configure controls

  var positionTargetCamera = positionCamera.clone();
  positionTargetCamera.setAltitude(0);

  if (options.noControls) {
    this.camera.setPosition(positionCamera);
    this.camera.camera3D.lookAt(positionTargetCamera.as('EPSG:4978').xyz());
  } else {
    this.controls = new _GlobeControls.default(this, positionTargetCamera, positionCamera.altitude(), _Coordinates.ellipsoidSizes.x);
    this.controls.handleCollision = typeof options.handleCollision !== 'undefined' ? options.handleCollision : true;
  }

  var mfogDistance = _Coordinates.ellipsoidSizes.x * 160.0;
  this._fullSizeDepthBuffer = null;
  var renderer = this.mainLoop.gfxEngine.renderer;
  var coordCam = new _Coordinates.default(this.referenceCrs, 0, 0, 0);
  var coordGeoCam = new _Coordinates.C.EPSG_4326();
  var skyBaseColor = new THREE.Color(0x93d5f8);
  var colorSky = new THREE.Color();
  var spaceColor = new THREE.Color(0x030508);
  var limitAlti = 600000;
  this.addFrameRequester(_MainLoop.MAIN_LOOP_EVENTS.BEFORE_RENDER, function () {
    if (_this._fullSizeDepthBuffer != null) {
      // clean depth buffer
      _this._fullSizeDepthBuffer = null;
    }
  });
  this.addFrameRequester(_MainLoop.MAIN_LOOP_EVENTS.AFTER_CAMERA_UPDATE, function () {
    var v = new THREE.Vector3();
    v.setFromMatrixPosition(tileLayer.object3d.matrixWorld);
    var len = v.distanceTo(_this.camera.camera3D.position);
    v.setFromMatrixScale(tileLayer.object3d.matrixWorld); // Compute fog distance, this function makes it possible to have a shorter distance
    // when the camera approaches the ground

    _this.fogDistance = mfogDistance * Math.pow((len - _Coordinates.ellipsoidSizes.x * 0.99) * 0.25 / _Coordinates.ellipsoidSizes.x, 1.5); // get altitude camera

    coordCam.set(_this.referenceCrs, _this.camera.camera3D.position).as('EPSG:4326', coordGeoCam);
    var altitude = coordGeoCam.altitude(); // If the camera altitude is below limitAlti,
    // we interpolate between the sky color and the space color

    if (altitude < limitAlti) {
      colorSky.copy(spaceColor).lerp(skyBaseColor, (limitAlti - altitude) / limitAlti);
      renderer.setClearColor(colorSky, renderer.getClearAlpha());
    } else {
      renderer.setClearColor(spaceColor, renderer.getClearAlpha());
    }
  });
  this.tileLayer = tileLayer;

  var fn = function () {
    _this.removeEventListener(_View.VIEW_EVENTS.LAYERS_INITIALIZED, fn);

    _this.dispatchEvent({
      type: GLOBE_VIEW_EVENTS.GLOBE_INITIALIZED
    });
  }; // GlobeView needs this.camera.resize to set perpsective matrix camera


  this.camera.resize(viewerDiv.clientWidth, viewerDiv.clientHeight);
  this.addEventListener(_View.VIEW_EVENTS.LAYERS_INITIALIZED, fn);
}

GlobeView.prototype = Object.create(_View.default.prototype);
GlobeView.prototype.constructor = GlobeView;

GlobeView.prototype.addLayer = function (layer) {
  if (!layer) {
    return new Promise(function (resolve, reject) {
      return reject(new Error('layer is undefined'));
    });
  }

  if (layer.isColorLayer) {
    var colorLayerCount = this.getLayers(function (l) {
      return l.isColorLayer;
    }).length;
    layer.sequence = colorLayerCount;
  } else if (layer.isElevationLayer) {
    if (layer.source.isWMTSSource && layer.source.tileMatrixSet !== 'WGS84G') {
      throw new Error('Only WGS84G tileMatrixSet is currently supported for WMTS elevation layers');
    }
  }

  var layerId = layer.id;

  var layerPromise = _View.default.prototype.addLayer.call(this, layer, this.tileLayer);

  this.dispatchEvent({
    type: GLOBE_VIEW_EVENTS.LAYER_ADDED,
    layerId: layerId
  });
  return layerPromise;
};
/**
 * Removes a specific imagery layer from the current layer list. This removes layers inserted with attach().
 * @example
 * view.removeLayer('layerId');
 * @param      {string}   layerId      The identifier
 * @return     {boolean}
 */


GlobeView.prototype.removeLayer = function (layerId) {
  if (_View.default.prototype.removeLayer.call(this, layerId)) {
    this.dispatchEvent({
      type: GLOBE_VIEW_EVENTS.LAYER_REMOVED,
      layerId: layerId
    });
    return true;
  }
};

GlobeView.prototype.setRealisticLightingOn = function (value) {
  var coSun = _CoordStars.default.getSunPositionInScene(new Date().getTime(), 48.85, 2.35).normalize();

  this.lightingPos = coSun.normalize();
  var lighting = this.tileLayer.lighting;
  lighting.enable = value;
  lighting.position = coSun;
  this.atmosphere.setRealisticOn(value);
  this.atmosphere.updateLightingPos(coSun);
  this.updateMaterialProperty('lightingEnabled', value);
  this.updateMaterialProperty('lightPosition', coSun);
  this.notifyChange(this.tileLayer);
};

GlobeView.prototype.setLightingPos = function (pos) {
  var lightingPos = pos || _CoordStars.default.getSunPositionInScene(this.ellipsoid, new Date().getTime(), 48.85, 2.35);

  this.updateMaterialProperty('lightPosition', lightingPos.clone().normalize());
  this.notifyChange(this.tileLayer);
};

GlobeView.prototype.updateMaterialProperty = function (property, value) {
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = this.tileLayer.level0Nodes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var n = _step.value;
      n.traverse(function (obj) {
        if (obj.material && obj.material[property] !== undefined) {
          obj.material[property] = value;
        }
      });
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
};

var _default = GlobeView;
exports.default = _default;