"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateLayeredMaterialNodeImagery = updateLayeredMaterialNodeImagery;
exports.updateLayeredMaterialNodeElevation = updateLayeredMaterialNodeElevation;
exports.removeLayeredMaterialNodeLayer = removeLayeredMaterialNodeLayer;

var _LayerUpdateStrategy = require("../Layer/LayerUpdateStrategy");

var _LayerUpdateState = _interopRequireDefault(require("../Layer/LayerUpdateState"));

var _Layer = require("../Layer/Layer");

var _CancelledCommandException = _interopRequireDefault(require("../Core/Scheduler/CancelledCommandException"));

var _OGCWebServiceHelper = require("../Provider/OGCWebServiceHelper");

var _XbilParser = require("../Parser/XbilParser");

// max retry loading before changing the status to definitiveError
var MAX_RETRY = 4;

function getSourceExtent(node, extent, targetLevel, source) {
  // source.getSourceExtents is to StaticSource
  if (source && source.getSourceExtents) {
    var ext = source.getSourceExtents(extent).extent;
    ext.zoom = node.level;
    return ext;
  } else if (extent.isTiledCrs()) {
    return extent.extentParent(targetLevel);
  } else {
    var parent = node.findAncestorFromLevel(targetLevel);

    if (parent.extent) {
      // Needed to initNodeElevationTextureFromParent, insertSignificantValuesFromParent,
      // isColorLayerDownscaled
      // Must be removed
      parent.extent.zoom = parent.level;
    }

    return parent.extent;
  }
}

function materialCommandQueuePriorityFunction(material) {
  // We know that 'node' is visible because commands can only be
  // issued for visible nodes.
  // TODO: need priorization of displayed nodes
  if (material.visible) {
    // Then prefer displayed node over non-displayed one
    return 100;
  } else {
    return 10;
  }
}

function refinementCommandCancellationFn(cmd) {
  if (!cmd.requester.parent || !cmd.requester.material) {
    return true;
  }

  if (cmd.force) {
    return false;
  } // Cancel the command if the tile already has a better texture.
  // This is only needed for elevation layers, because we may have several
  // concurrent layers but we can only use one texture.


  if (cmd.layer.isElevationLayer && cmd.requester.material.getElevationLayer() && cmd.targetLevel <= cmd.requester.material.getElevationLayer().level) {
    return true;
  }

  return !cmd.requester.material.visible;
}

function updateLayeredMaterialNodeImagery(context, layer, node, parent) {
  var material = node.material;

  if (!parent || !material) {
    return;
  }

  var extentsDestination = node.getCoordsForSource(layer.source);
  var nodeLayer = material.getLayer(layer.id); // Initialisation

  if (node.layerUpdateState[layer.id] === undefined) {
    node.layerUpdateState[layer.id] = new _LayerUpdateState.default();

    if (!layer.source.extentsInsideLimit(extentsDestination)) {
      // we also need to check that tile's parent doesn't have a texture for this layer,
      // because even if this tile is outside of the layer, it could inherit it's
      // parent texture
      if (!(!layer.noTextureParentOutsideLimit && parent.material && parent.material.getLayer && parent.material.getLayer(layer.id))) {
        node.layerUpdateState[layer.id].noMoreUpdatePossible();
        return;
      }
    }

    if (!nodeLayer) {
      // Create new MaterialLayer
      var tileMT = layer.options.tileMatrixSet || extentsDestination[0].crs();
      nodeLayer = material.addLayer(layer, tileMT); // TODO: Sequence must be returned by parent geometry layer

      var colorLayers = context.view.getLayers(function (l) {
        return l.isColorLayer;
      });

      var sequence = _Layer.ImageryLayers.getColorLayersIdOrderedBySequence(colorLayers);

      material.setSequence(sequence); // Init the new MaterialLayer by parent

      var parentLayer = parent.material && parent.material.getLayer(layer.id);
      nodeLayer.initFromParent(parentLayer, extentsDestination);
    } // Proposed new process, two separate processes:
    //      * FIRST PASS: initNodeXXXFromParent and get out of the function
    //      * SECOND PASS: Fetch best texture
    // The two-step allows you to filter out unnecessary requests
    // Indeed in the second pass, their state (not visible or not displayed) can block them to fetch


    if (nodeLayer.level >= layer.source.zoom.min) {
      context.view.notifyChange(node, false);
      return;
    }
  } // Node is hidden, no need to update it


  if (!material.visible) {
    return;
  } // TODO: move this to defineLayerProperty() declaration
  // to avoid mixing layer's network updates and layer's params
  // Update material parameters


  if (nodeLayer) {
    nodeLayer.visible = layer.visible;
    nodeLayer.opacity = layer.opacity;
  }

  var ts = Date.now(); // An update is pending / or impossible -> abort

  if (!layer.visible || !node.layerUpdateState[layer.id].canTryUpdate(ts)) {
    return;
  }

  if (layer.source.canTileTextureBeImproved) {
    // if the layer has a custom method -> use it
    if (!layer.source.canTileTextureBeImproved(node.extent, nodeLayer.textures[0])) {
      node.layerUpdateState[layer.id].noMoreUpdatePossible();
      return;
    }
  } else if (nodeLayer.level >= node.getZoomForLayer(layer)) {
    // default decision method
    node.layerUpdateState[layer.id].noMoreUpdatePossible();
    return;
  } // is fetching data from this layer disabled?


  if (layer.frozen) {
    return;
  }

  var failureParams = node.layerUpdateState[layer.id].failureParams;
  var destinationLevel = extentsDestination[0].zoom || node.level;
  var targetLevel = (0, _LayerUpdateStrategy.chooseNextLevelToFetch)(layer.updateStrategy.type, node, destinationLevel, nodeLayer.level, layer, failureParams);

  if (targetLevel <= nodeLayer.level) {
    return;
  } // Get equivalent of extent destination in source


  var extentsSource = [];
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = extentsDestination[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var extentDestination = _step.value;
      var extentSource = getSourceExtent(node, extentDestination, targetLevel, layer.source);

      if (extentSource && !layer.source.extentInsideLimit(extentSource)) {
        // Retry extentInsideLimit because you must check with the targetLevel
        // if the first test extentInsideLimit returns that it is out of limits
        // and the node inherits from its parent, then it'll still make a command to fetch texture.
        node.layerUpdateState[layer.id].noMoreUpdatePossible();
        return;
      }

      extentsSource.push(extentSource);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  node.layerUpdateState[layer.id].newTry();
  var command = {
    /* mandatory */
    view: context.view,
    layer: layer,
    extentsSource: extentsSource,
    extentsDestination: extentsDestination,
    parsedData: layer.source.parsedData,
    requester: node,
    priority: materialCommandQueuePriorityFunction(material),
    earlyDropFunction: refinementCommandCancellationFn
  };
  return context.scheduler.execute(command).then(function (result) {
    if (material === null) {
      return;
    }

    var pitchs = extentsDestination.map(function (ext, i) {
      return ext.offsetToParent(extentsSource[i]);
    });

    if (result) {
      nodeLayer.setTextures(result, pitchs);
    }

    node.layerUpdateState[layer.id].success();
    return result;
  }, function (err) {
    if (err instanceof _CancelledCommandException.default) {
      node.layerUpdateState[layer.id].success();
    } else {
      var definitiveError = node.layerUpdateState[layer.id].errorCount > MAX_RETRY;
      node.layerUpdateState[layer.id].failure(Date.now(), definitiveError, {
        targetLevel: targetLevel
      });

      if (!definitiveError) {
        window.setTimeout(function () {
          context.view.notifyChange(node, false);
        }, node.layerUpdateState[layer.id].secondsUntilNextTry() * 1000);
      }
    }
  });
}

function updateLayeredMaterialNodeElevation(context, layer, node, parent) {
  var material = node.material;

  if (!parent || !material) {
    return;
  } // TODO: we need either
  //  - compound or exclusive layers
  //  - support for multiple elevation layers
  // Elevation is currently handled differently from color layers.
  // This is caused by a LayeredMaterial limitation: only 1 elevation texture
  // can be used (where a tile can have N textures x M layers)


  var extentsDestination = node.getCoordsForSource(layer.source);
  var tileMT = layer.options.tileMatrixSet || extentsDestination[0].crs(); // Init elevation layer, and inherit from parent if possible

  var nodeLayer = material.getElevationLayer();

  if (!nodeLayer) {
    nodeLayer = material.addLayer(layer, tileMT);
    material.setSequenceElevation(layer.id);
  }

  if (node.layerUpdateState[layer.id] === undefined) {
    node.layerUpdateState[layer.id] = new _LayerUpdateState.default();
    var parentLayer = parent.material && parent.material.getLayer(layer.id);
    nodeLayer.initFromParent(parentLayer, extentsDestination); // If the texture resolution has a poor precision for this node, we don't
    // extract min-max from the texture (too few information), we instead chose
    // to use parent's min-max.

    var useMinMaxFromParent = extentsDestination[0].zoom - nodeLayer.zoom > 6;

    if (nodeLayer.textures[0]) {
      if (!useMinMaxFromParent) {
        var _computeMinMaxElevati = (0, _XbilParser.computeMinMaxElevation)(nodeLayer.textures[0].image.data, _OGCWebServiceHelper.SIZE_TEXTURE_TILE, _OGCWebServiceHelper.SIZE_TEXTURE_TILE, nodeLayer.offsetScales[0]),
            min = _computeMinMaxElevati.min,
            max = _computeMinMaxElevati.max;

        node.setBBoxZ(min, max);
      }
    }

    if (nodeLayer.level >= layer.source.zoom.min) {
      context.view.notifyChange(node, false);
      return;
    }
  } // Try to update


  var ts = Date.now(); // Possible conditions to *not* update the elevation texture

  if (layer.frozen || !material.visible || !node.layerUpdateState[layer.id].canTryUpdate(ts)) {
    return;
  } // Does this tile needs a new texture?


  if (layer.source.canTileTextureBeImproved) {
    // if the layer has a custom method -> use it
    if (!layer.source.canTileTextureBeImproved(layer, nodeLayer.textures[0])) {
      node.layerUpdateState[layer.id].noMoreUpdatePossible();
      return;
    }
  }

  var targetLevel = (0, _LayerUpdateStrategy.chooseNextLevelToFetch)(layer.updateStrategy.type, node, extentsDestination[0].zoom, nodeLayer.level, layer);

  if (targetLevel <= nodeLayer.level) {
    node.layerUpdateState[layer.id].noMoreUpdatePossible();
    return Promise.resolve();
  }

  var extentsSource = [];
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = extentsDestination[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var nodeExtent = _step2.value;
      var extentSource = getSourceExtent(node, nodeExtent, targetLevel);

      if (extentSource && !layer.source.extentInsideLimit(extentSource)) {
        node.layerUpdateState[layer.id].noMoreUpdatePossible();
        return;
      }

      extentsSource.push(extentSource);
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  node.layerUpdateState[layer.id].newTry();
  var command = {
    /* mandatory */
    view: context.view,
    layer: layer,
    extentsSource: extentsSource,
    extentsDestination: extentsDestination,
    requester: node,
    priority: materialCommandQueuePriorityFunction(material),
    earlyDropFunction: refinementCommandCancellationFn
  };
  return context.scheduler.execute(command).then(function (textures) {
    var elevation = {
      texture: textures[0],
      pitch: extentsDestination[0].offsetToParent(extentsSource[0])
    }; // Do not apply the new texture if its level is < than the current
    // one.  This is only needed for elevation layers, because we may
    // have several concurrent layers but we can only use one texture.

    if (targetLevel <= nodeLayer.level) {
      node.layerUpdateState[layer.id].noMoreUpdatePossible();
      return;
    }

    node.layerUpdateState[layer.id].success();

    if (elevation.texture) {
      if (layer.useColorTextureElevation) {
        elevation.min = layer.colorTextureElevationMinZ;
        elevation.max = layer.colorTextureElevationMaxZ;
      } else {
        var _computeMinMaxElevati2 = (0, _XbilParser.computeMinMaxElevation)(elevation.texture.image.data, _OGCWebServiceHelper.SIZE_TEXTURE_TILE, _OGCWebServiceHelper.SIZE_TEXTURE_TILE, elevation.pitch),
            _min = _computeMinMaxElevati2.min,
            _max = _computeMinMaxElevati2.max;

        elevation.min = !_min ? 0 : _min;
        elevation.max = !_max ? 0 : _max;
      }
    }

    node.setBBoxZ(elevation.min, elevation.max);
    nodeLayer = material.getLayer(layer.id);

    if (!nodeLayer) {
      nodeLayer = material.addLayer(layer, tileMT);
    }

    material.setSequenceElevation(layer.id);
    nodeLayer.setTexture(0, elevation.texture, elevation.pitch);
    var nodeParent = parent.material && parent.material.getElevationLayer();
    nodeLayer.replaceNoDataValueFromParent(nodeParent, layer.noDataValue);
  }, function (err) {
    if (err instanceof _CancelledCommandException.default) {
      node.layerUpdateState[layer.id].success();
    } else {
      var definitiveError = node.layerUpdateState[layer.id].errorCount > MAX_RETRY;
      node.layerUpdateState[layer.id].failure(Date.now(), definitiveError);

      if (!definitiveError) {
        window.setTimeout(function () {
          context.view.notifyChange(node, false);
        }, node.layerUpdateState[layer.id].secondsUntilNextTry() * 1000);
      }
    }
  });
}

function removeLayeredMaterialNodeLayer(layerId) {
  return function (node) {
    if (node.material && node.material.removeLayer) {
      node.material.removeLayer(layerId);

      if (node.material.elevationLayerIds[0] == layerId) {
        node.setBBoxZ(0, 0);
      }
    }

    if (node.layerUpdateState && node.layerUpdateState[layerId]) {
      delete node.layerUpdateState[layerId];
    }
  };
}