"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.supportedFetchers = void 0;

var _GeoJsonParser = _interopRequireDefault(require("../Parser/GeoJsonParser"));

var _VectorTileParser = _interopRequireDefault(require("../Parser/VectorTileParser"));

var _Fetcher = _interopRequireDefault(require("./Fetcher"));

var _Cache = _interopRequireDefault(require("../Core/Scheduler/Cache"));

var _CancelledCommandException = _interopRequireDefault(require("../Core/Scheduler/CancelledCommandException"));

var supportedFetchers = new Map([['image/x-bil;bits=32', _Fetcher.default.textureFloat], ['geojson', _Fetcher.default.json], ['application/json', _Fetcher.default.json], ['application/x-protobuf;type=mapbox-vector', _Fetcher.default.arrayBuffer]]);
exports.supportedFetchers = supportedFetchers;
var supportedParsers = new Map([['geojson', _GeoJsonParser.default.parse], ['application/json', _GeoJsonParser.default.parse], ['application/x-protobuf;type=mapbox-vector', _VectorTileParser.default.parse]]);

function isValidData(data, extentDestination, validFn) {
  if (data && (!validFn || validFn(data, extentDestination))) {
    return data;
  }
}

var error = function (err, url, source) {
  source.handlingError(err, url);
  throw err;
};

function FetchAndConvertSourceData(url, layer, extentSource, extentDestination) {
  var source = layer.source;

  var fetcher = source.fetcher || supportedFetchers.get(source.format) || _Fetcher.default.texture;

  var parser = source.parser || supportedParsers.get(source.format) || function (data) {
    return data;
  };

  var parsingOptions = {
    buildExtent: !layer.isGeometryLayer,
    crsIn: source.projection,
    crsOut: layer.projection,
    // TODO FIXME: error in filtering vector tile
    // filteringExtent: extentDestination.as(layer.projection),
    filteringExtent: layer.isGeometryLayer ? extentDestination : undefined,
    filter: layer.filter,
    isInverted: source.isInverted,
    mergeFeatures: layer.mergeFeatures === undefined ? true : layer.mergeFeatures,
    withNormal: layer.isGeometryLayer,
    withAltitude: layer.isGeometryLayer
  }; // Fetch data

  return fetcher(url, source.networkOptions).then(function (fetchedData) {
    fetchedData.coords = extentSource; // Parse fetched data, it parses file to itowns's object

    return parser(fetchedData, parsingOptions);
  }, function (err) {
    return error(err, url, source);
  }).then(function (parsedData) {
    return (// Convert parsed data, it converts itowns's object to THREE's object
      layer.convert(parsedData, extentDestination)
    );
  }, function (err) {
    return error(err, url, source);
  });
}

var _default = {
  executeCommand: function executeCommand(command) {
    var promises = [];
    var layer = command.layer;
    var source = layer.source;
    var requester = command.requester;
    var extentsSource = command.extentsSource;
    var extentsDestination = command.extentsDestination || extentsSource;
    var parsedData = command.parsedData || []; // TODO: Find best place to cancel Command

    if (requester && !requester.material) {
      // request has been deleted
      return Promise.reject(new _CancelledCommandException.default(command));
    }

    for (var i = 0, max = extentsSource.length; i < max; i++) {
      var extSource = extentsSource[i];
      var extDest = extentsDestination[i]; // If source, we must fetch and convert data
      // URL of the resource you want to fetch

      var url = source.urlFromExtent(extSource); // Already fetched and parsed data that can be used

      var validedParsedData = isValidData(parsedData[i], extDest, layer.isValidData) || source.parsedData; // Tag to Cache data

      var tag = validedParsedData ? "".concat(url, ",").concat(extDest.toString(',')) : url; // Get converted source data, in cache

      var convertedSourceData = _Cache.default.get(tag); // If data isn't in cache


      if (!convertedSourceData) {
        if (validedParsedData) {
          // Use parsed data
          convertedSourceData = layer.convert(validedParsedData, extDest, layer);
        } else {
          // Fetch and convert
          convertedSourceData = FetchAndConvertSourceData(url, layer, extSource, extDest);
        } // Put converted data in cache


        _Cache.default.set(tag, convertedSourceData, _Cache.default.POLICIES.TEXTURE);
      } // Verify some command is resolved
      // See old WFSProvider : command.resolve(result)


      promises.push(convertedSourceData);
    }

    return Promise.all(promises);
  }
};
exports.default = _default;